/****************************************************************************
 * interpreters/micropython/mpconfigport.h
 *
 *   Copyright (C) 2015 Gregory Nutt. All rights reserved.
 *   Authors: Dave Marples <dave@marples.net>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 3. Neither the name NuttX nor the names of its contributors may be
 *    used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 ****************************************************************************/

#ifndef __APPS_INTERPRETERS_MICROPYTHON_MPCONFIGPORT_H
#define __APPS_INTERPRETERS_MICROPYTHON_MPCONFIGPORT_H 1

/****************************************************************************
 * Included Files
 ****************************************************************************/

#include <stdint.h>

/* We need to provide a declaration/definition of alloca() */

#include <alloca.h>

/****************************************************************************
 * Pre-processor Definitions
 ****************************************************************************/

/* options to control how Micro Python is built */

#define MICROPY_ALLOC_PATH_MAX         (512)
#define MICROPY_EMIT_X64               (0)
#define MICROPY_EMIT_THUMB             (0)
#define MICROPY_EMIT_INLINE_THUMB      (0)
#define MICROPY_MEM_STATS              (0)
#define MICROPY_DEBUG_PRINTERS         (0)
#define MICROPY_ENABLE_GC              (0)
#define MICROPY_HELPER_REPL            (1)
#define MICROPY_HELPER_LEXER_UNIX      (0)
#define MICROPY_ENABLE_SOURCE_LINE     (0)
#define MICROPY_ENABLE_DOC_STRING      (0)
#define MICROPY_ERROR_REPORTING        (MICROPY_ERROR_REPORTING_TERSE)
#define MICROPY_PY_BUILTINS_BYTEARRAY  (0)
#define MICROPY_PY_BUILTINS_MEMORYVIEW (0)
#define MICROPY_PY_BUILTINS_FROZENSET  (0)
#define MICROPY_PY_BUILTINS_SET        (0)
#define MICROPY_PY_BUILTINS_SLICE      (0)
#define MICROPY_PY_BUILTINS_PROPERTY   (0)
#define MICROPY_PY___FILE__            (0)
#define MICROPY_PY_GC                  (0)
#define MICROPY_PY_ARRAY               (0)
#define MICROPY_PY_COLLECTIONS         (0)
#define MICROPY_PY_MATH                (0)
#define MICROPY_PY_CMATH               (0)
#define MICROPY_PY_IO                  (0)
#define MICROPY_PY_STRUCT              (0)
#define MICROPY_PY_SYS                 (0)
#define MICROPY_CPYTHON_COMPAT         (0)
#define MICROPY_LONGINT_IMPL           (MICROPY_LONGINT_IMPL_MPZ)
#define MICROPY_FLOAT_IMPL             (MICROPY_FLOAT_IMPL_FLOAT)

/* type definitions for the specific machine */

#define BYTES_PER_WORD                 (4)

#define MICROPY_MAKE_POINTER_CALLABLE(p) ((void*)((mp_uint_t)(p) | 1))

#define UINT_FMT                       "%lu"
#define INT_FMT                        "%ld"

#define MICROPY_PORT_BUILTINS \
    { MP_OBJ_NEW_QSTR(MP_QSTR_open), (mp_obj_t)&mp_builtin_open_obj },

/****************************************************************************
 * Public Types
 ****************************************************************************/

typedef intptr_t mp_int_t;               /* must be pointer size */
typedef uintptr_t mp_uint_t;             /* must be pointer size */
typedef void *machine_ptr_t;             /* must be of pointer size */
typedef const void *machine_const_ptr_t; /* must be of pointer size */
typedef long mp_off_t;

/****************************************************************************
 * Public Data
 ****************************************************************************/

/* extra built in names to add to the global namespace */

extern const struct _mp_obj_fun_builtin_t mp_builtin_open_obj;

#endif /* __APPS_INTERPRETERS_MICROPYTHON_MPCONFIGPORT_H */
